==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2206_162.zip: ronportal/ron/db/sqldialectmssql.php <==
<?php
/**
 * stanislav.elbl, 23.09.2021 13:44
 */

namespace Ron\DB;

class SqlDialectMssql implements SqlDialect
{
    public function getDbType()
    {
        return 'mssql';
    }


    public function getPdoParameterLobOptions()
    {
        return \PDO::SQLSRV_ENCODING_BINARY;
    }


    public function extractYearSQL($text)
    {
        return "YEAR($text)";
    }

    /**
     * Prevede hodnotu datoveho pole nacteneho z databaze na timestamp
     * @param mixed $data
     * @return int|null
     */
    public function dataToTimestamp($data)
    {
        if (!isset($data)) {
            return null;
        }

        $onlyTime = strstr($data, '1899') !== false;
        //$str = str_replace('1899', '1970', $str); // pokud je tam 1899, tak je to nejspis jenom cas - prevede ho - finta, aby se cas dobre zobrazoval

//        // TODO: omezit tento test na PHP_INT_SIZE === 4 ?
//        if (preg_match ("/2(\d\d\d)/", $data, $m) && ($m[1] >= 38)) {
//            // maximalni hodnota timestampu je do roku 2038 !
//            return null;
//        }

        if (!$onlyTime) {
            $result = strtotime($data);
            if ($result != -1) {
                return $result; // - $basetime;
            }
        }

        $pomarray = explode(" ", $data); // rozdeli datum a cas
        $datearray = $onlyTime ? [1970, 1, 1] : explode("-", $pomarray[0]); // rozdeli slozky data
        $timearray1 = explode(".", $pomarray[1]); // oddeli setiny od casu
        $timearray = explode(":", $timearray1[0]); // rozdeli slozky casu

        $result = mktime($timearray[0], $timearray[1], $timearray[2], $datearray[1], $datearray[2], $datearray[0]);

        return $result; //  - $basetime;
    }


    /**
     * Vraci SQL konstantu pro zadany cas (unix timestamp)
     * @param int $timestamp
     * @return string
     */
    public function timestampToSQL($timestamp)
    {

        if (!isset($timestamp)) {
            return 'null';
        }

        return "{ts '" . date('Y-m-d H:i:s', $timestamp) . "'}";
    }


    /**
     * Vraci SQL konstantu pro zadany datum (unix timestamp) - cas ignoruje
     * @param int $timestamp
     * @return string
     */
    public function dateToSQL($timestamp)
    {
        if (!isset($timestamp)) {
            return 'null';
        }

        return "{d '" . date('Y-m-d', $timestamp) . "'}";
    }


    /**
     * Vraci SQL konstantu pro zadany cas (unix timestamp) bez datumu
     * @param int $timestamp
     * @return string
     */
    public function timeToSQL($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
            $timestamp = $timestamp - 3600 * $h;
            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);
            return "{ ts '1899-12-30 $h:$m:$s'}";
        }

        return "{ ts '1899-12-30 " . date('H:i:s', $timestamp) . "'}";
    }


    /**
     * Vraci SQL konstantu pro vlozeni binarnich dat
     * @param $text
     * @return mixed
     */
    public function stringToBinarySQL($text)
    {

        return pack('H*', $text);
    }

}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2302_169.zip: ronportal/ron/db/sqldialectmssql.php <==
<?php
/**
 * stanislav.elbl, 23.09.2021 13:44
 */

namespace Ron\DB;

class SqlDialectMssql implements SqlDialect
{
    public function getDbType()
    {
        return 'mssql';
    }


    public function getPdoParameterLobOptions()
    {
        return \PDO::SQLSRV_ENCODING_BINARY;
    }


    public function getConcatenateOperator()
    {
        return '+';
    }


    public function getReplaceFunctions()
    {
        return [];
    }


    public function extractYearSQL($text)
    {
        return "YEAR($text)";
    }

    /**
     * Prevede hodnotu datoveho pole nacteneho z databaze na timestamp
     * @param mixed $data
     * @return int|null
     */
    public function dataToTimestamp($data)
    {
        if (!isset($data)) {
            return null;
        }

        $onlyTime = strstr($data, '1899') !== false;
        //$str = str_replace('1899', '1970', $str); // pokud je tam 1899, tak je to nejspis jenom cas - prevede ho - finta, aby se cas dobre zobrazoval

//        // TODO: omezit tento test na PHP_INT_SIZE === 4 ?
//        if (preg_match ("/2(\d\d\d)/", $data, $m) && ($m[1] >= 38)) {
//            // maximalni hodnota timestampu je do roku 2038 !
//            return null;
//        }

        if (!$onlyTime) {
            $result = strtotime($data);
            if ($result != -1) {
                return $result; // - $basetime;
            }
        }

        $pomarray = explode(" ", $data); // rozdeli datum a cas
        $datearray = $onlyTime ? [1970, 1, 1] : explode("-", $pomarray[0]); // rozdeli slozky data
        $timearray1 = explode(".", $pomarray[1]); // oddeli setiny od casu
        $timearray = explode(":", $timearray1[0]); // rozdeli slozky casu

        $result = mktime($timearray[0], $timearray[1], $timearray[2], $datearray[1], $datearray[2], $datearray[0]);

        return $result; //  - $basetime;
    }


    /**
     * Vraci SQL konstantu pro zadany cas (unix timestamp)
     * @param int $timestamp
     * @return string
     */
    public function timestampToSQL($timestamp)
    {

        if (!isset($timestamp)) {
            return 'null';
        }

        return "{ts '" . date('Y-m-d H:i:s', $timestamp) . "'}";
    }


    /**
     * Vraci SQL konstantu pro zadany datum (unix timestamp) - cas ignoruje
     * @param int $timestamp
     * @return string
     */
    public function dateToSQL($timestamp)
    {
        if (!isset($timestamp)) {
            return 'null';
        }

        return "{d '" . date('Y-m-d', $timestamp) . "'}";
    }


    /**
     * Vraci SQL konstantu pro zadany cas (unix timestamp) bez datumu
     * @param int $timestamp
     * @return string
     */
    public function timeToSQL($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
            $timestamp = $timestamp - 3600 * $h;
            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);
            return "{ ts '1899-12-30 $h:$m:$s'}";
        }

        return "{ ts '1899-12-30 " . date('H:i:s', $timestamp) . "'}";
    }


    /**
     * Vraci SQL vyraz pro zjisteni nove hodnoty pro zadanou sekvenci
     * @param string $sequenceName
     * @param bool $standalone  Urcuje, jestli se jedna o samostatny vyraz pro zjisteni nove hodnoty, nebo je to soucast jineho vyrazu (napr. INSERT...)
     * @return string
     */
    public function nextSequenceValueSQL(string $sequenceName, $standalone = true): string
    {
        return "NEXT VALUE FOR $sequenceName";
    }


    /**
     * Vraci SQL dotaz pro zjisteni, jestli existuje sekvence zadaneho jmena.
     * @param string $sequenceName
     * @return string
     */
    public function sequenceExistsSQL(string $sequenceName): string
    {
        return
            'SELECT name FROM sys.sequences' .
            ' WHERE UPPER(name) = ' . "'" . strtoupper($sequenceName) . "'";
    }






//    /**
//     * Vraci SQL konstantu pro vlozeni binarnich dat
//     * @param $text
//     * @return mixed
//     */
//    public function stringToBinarySQL($text)
//    {
//
//        return pack('H*', $text);
//    }

}
